/**
 * @File
 *
 * Javascript handler for livetracking map.
 *
 * Prepare the map, receive a message from Nodejs and add the position.
 * Toggle connected / disconnected.
 * Show last position.
 */

(function($) {
  var ol;
  var pointFeature;
  var pointsArray = Array();
  var line;
  var lineFeature;
  var positionLayer;
  var trackLayer;
  var autoZoom = true;
  var lastTime;

  Drupal.behaviors.nodejs_livetracking = {
    'attach': function(context, setting) {
      if (context == document) {
        ol = $('#openlayers-map').data('openlayers');
        // Catch defined 'livetracking_style' from the api.
        var mapStyle = new OpenLayers.StyleMap(ol.map.styles);
        // Create position and track layer and assigning a livetracking style. to it
        positionLayer = new OpenLayers.Layer.Vector('User position', {styleMap: mapStyle});
        trackLayer = new OpenLayers.Layer.Vector('User track', {styleMap: mapStyle});
        // Add the layer to the map.
        ol.openlayers.addLayer(positionLayer);
        ol.openlayers.addLayer(trackLayer);

        var point = new OpenLayers.Geometry.Point(0, 0).transform("EPSG:4326", ol.map.projection);
        pointFeature = new OpenLayers.Feature.Vector(point);
        positionLayer.addFeatures([pointFeature]);

        if (pointsArray.length > 2) {
          ol.openlayers.zoomToExtent(trackLayer.getDataExtent());
        }

        $('#followme').bind('change', function() {
          autoZoom = !autoZoom;
        });

        setInterval(function() {
          $('#timestamp').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime))
        }, 5000);
      }
     },

    // Get latest marker.
    'getPoint' : function() {
      return pointFeature;
    },

    'storePoint' : function(point) {
      pointsArray.push(point);
      if (pointsArray.length == 2) {
        line = new OpenLayers.Geometry.LineString(pointsArray);
        lineFeature = new OpenLayers.Feature.Vector(line);
        trackLayer.addFeatures([lineFeature]);
      }
      else if (pointsArray.length > 2) {
        line.addPoint(point);
      }
      trackLayer.redraw();
    },

    // Set latest marker.
    'setPoint' : function(feature) {
      pointFeature = feature;
    },

    'getLine' : function() {
      return lineFeature;
    },

    'setLine' : function(feature) {
      lineFeature = feature
    },

    // Zoom the feature (Follow).
    'zoomToFeature' : function() {
      if (autoZoom == true){
        ol.openlayers.zoomToExtent(positionLayer.getDataExtent());
      }
    },

    'setPresence' : function(bool) {
      if (bool == true) {
        $("#connected").removeClass('hidden');
        $("#disconnected").addClass('hidden');
      }
      else {
        $("#connected").addClass('hidden');
        $("#disconnected").removeClass('hidden');
      }
    },

    'updateInfo' : function(message) {
      for (var key in message) {
        if (['lat', 'lon', 'precision', 'vario', 'alt', 'sog'].indexOf(key) > -1) {
          $('#' + key).html(message[key]);
        }
      }
    },

    'setLastTime' : function(time) {
      lastTime = time;
       $('#tm').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime));
    },

    // Return time ago from timestamp. Copied and adapted from : https://gist.github.com/skattyadz/1285806
    'timeAgo' : function(time) {
      if (time == undefined) {
        return Drupal.t("Unknown");
      }
      var units = [
        { name: Drupal.t("second"), limit: 60, in_seconds: 1 },
        { name: Drupal.t("minute"), limit: 3600, in_seconds: 60 },
        { name: Drupal.t("hour"), limit: 86400, in_seconds: 3600 },
        { name: Drupal.t("day"), limit: 604800, in_seconds: 86400 },
        { name: Drupal.t("week"), limit: 2629743, in_seconds: 604800 },
        { name: Drupal.t("month"), limit: 31556926, in_seconds: 2629743 },
        { name: Drupal.t("year"), limit: null, in_seconds: 31556926 }
      ];

      var diff = (new Date() - new Date(time)) / 1000;
      if (diff < 5) {
        return Drupal.t("just now");
      }
      var i = 0;
      while (unit = units[i++]) {
        if (diff < unit.limit || !unit.limit){
          var diff = Math.floor(diff / unit.in_seconds);
          return diff + " " + unit.name + (diff > 1 ? "s" : "") + " " + Drupal.t("old");
        }
      };
    },
  };

  Drupal.Nodejs.callbacks.nodejs_livetracking_tracking = {
    callback: function (message) {
      if (message.destinationChannel != "all_tracking") {
        if (message.messageType == 'device-auth') {
          Drupal.behaviors.nodejs_livetracking.setPresence(true);
          message['data'] = {
            'body' : Drupal.t('User @id is now here !', {'@id' : message.uid}),
            'subject' : Drupal.t('Device connected')
          };
          Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
        }
        else if (message.messageType == 'device-off') {
          Drupal.behaviors.nodejs_livetracking.setPresence(false);
          message['data'] = {
            'body' : Drupal.t('We just lost user @id !', {'@id' : message.uid}),
            'subject' : Drupal.t('Device disconnected')
          };
          Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
          // There is no position in this message. I just quit now.
          return;
        }
        var newLonLat = new OpenLayers.LonLat(message.position.lon, message.position.lat).transform("EPSG:4326", ol.map.projection);
        pointFeature.move(newLonLat);
        positionLayer.redraw();
        Drupal.behaviors.nodejs_livetracking.zoomToFeature();
        var point = new OpenLayers.Geometry.Point(message.position.lon, message.position.lat).transform("EPSG:4326", ol.map.projection);
        Drupal.behaviors.nodejs_livetracking.storePoint(point);
        Drupal.behaviors.nodejs_livetracking.updateInfo(message.position);
        Drupal.behaviors.nodejs_livetracking.setLastTime(message.position.tm);
      }
    },
  };
})(jQuery);
